/* 
 * File:   charge_Fictive_main.c
 * Author: lectro-Bidouilleur
 *
 * Created on August 24, 2015, 12:14 PM
 */

#define _XTAL_FREQ 1000000 //The speed of your internal(or)external oscillator
#define USE_OR_MASKS   // Permet d'utiliser les masques OR prdfinis dans les fichiers header,
#include <xc.h>        // de la faon  MASK1 | MASK2 | MASK3...
#include <stdio.h>
#include <stdlib.h>
#include <pwm.h>
#include <timers.h>
#include <adc.h>

// CONFIGURATION BYTES
// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG1H
#pragma config OSC = INTIO2     // Oscillator Selection bits (Internal RC oscillator, port function on RA6 and port function on RA7)
#pragma config FSCM = OFF       // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF       // Internal External Switchover bit (Internal External Switchover mode disabled)
// CONFIG2L
#pragma config PWRT = OFF       // Power-up Timer Enable bit (PWRT disabled)
#pragma config BOR = OFF        // Brown-out Reset Enable bit (Brown-out Reset disabled)
// BORV = No Setting
// CONFIG2H
#pragma config WDT = OFF        // Watchdog Timer Enable bit (WDT disabled (control is placed on the SWDTEN bit))
#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)
// CONFIG3H
#pragma config MCLRE = OFF      // MCLR Pin Enable bit (RA5 input pin enabled, MCLR disabled)
// CONFIG4L
#pragma config STVR = ON        // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
#pragma config LVP = ON         // Low-Voltage ICSP Enable bit (Low-Voltage ICSP enabled)
// CONFIG5L
#pragma config CP0 = OFF        // Code Protection bit (Block 0 (00200-0007FFh) not code-protected)
#pragma config CP1 = OFF        // Code Protection bit (Block 1 (000800-000FFFh) not code-protected)
// CONFIG5H
#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot Block (000000-0001FFh) not code-protected)
#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)
// CONFIG6L
#pragma config WRT0 = OFF       // Write Protection bit (Block 0 (00200-0007FFh) not write-protected)
#pragma config WRT1 = OFF       // Write Protection bit (Block 1 (000800-000FFFh) not write-protected)
// CONFIG6H
#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) not write-protected)
#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot Block (000000-0001FFh) not write-protected)
#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write-protected)
// CONFIG7L
#pragma config EBTR0 = OFF      // Table Read Protection bit (Block 0 (00200-0007FFh) not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF      // Table Read Protection bit (Block 1 (000800-000FFFh) not protected from table reads executed in other blocks)
// CONFIG7H
#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot Block (000000-0001FFh) not protected from table reads executed in other blocks)


void main()
{
    // Configurer l'oscillateur interne  1MHz
    OSCCON = 0b01000010;

    // Cration de variables
    unsigned int Ibatt=0;           // Le courant mesur de la batterie
    unsigned int Ibatt_cible=0;     // Le courant cible  atteindre
    unsigned int Vbatt=0;           // La tension de la rsistance, ct batterie
    unsigned int Vmosfet=0;          // La tension de la rsistance, ct MOSFET
    unsigned int cycle_service=0;   // Le cycle de service
    unsigned int ADCResult=0;       // Le rsultat de la conversion ADC

    //----Initialiser l'ADC----
    #define adc_config1 (ADC_FOSC_2|ADC_RIGHT_JUST|ADC_2_TAD)    // La conversion prends 100us
    #define adc_config2 (ADC_CH0|ADC_INT_OFF|ADC_REF_VDD_VSS)
    #define adc_portconfig ADC_2ANA
    OpenADC(adc_config1,adc_config2,adc_portconfig);

//----Configurer le PWM ----
    #define config_timer2 (TIMER_INT_OFF|T2_PS_1_1)
    OpenTimer2(config_timer2);
    #define periode_pwm  0xFF
    OpenPWM1(periode_pwm);            //Configure PWM module and initialize PWM period

//----configurer la sortie du PWM----
    #define config_sortie SINGLE_OUT    // sortie unique
    #define mode_sortie PWM_MODE_1   // sortie active en niveau haut
    SetOutputPWM1(config_sortie, mode_sortie);    
    cycle_service = 614;        // On dmarre le PWM  60% de cycle sde service, ce qui donne environ 2V sur le MOSFET.
    Ibatt_cible =  102;         // Un courant cible de 1000mA pour cet essai. 2 ohms, diviseurs de 1/4 sur ADC
    //-----Boucle Principale-----
    while(1)    // Boucle Principale
    {
        SetDCPWM1(cycle_service);       // Assigner la valeur lue de l'ADC au cycle de service du PWM.
        __delay_ms(10);                 // Dlai pour stabiliser la sortie du PWM une fois filtre.

        //---Mersurer la tension de la batterie
        SetChanADC(ADC_CH0);    // Slectionner l'entre analogique AN0
        ConvertADC();           // Lancer la conversion analogique-numrique
        while(BusyADC());       // Attendre que l'ADC ait termin la conversion
        Vbatt = ReadADC();      // Lire le rsultat de la conversion

        //---Mersurer la tension du MOSFET
        SetChanADC(ADC_CH1);    // Slectionner l'entre analogique AN1
        ConvertADC();           // Lancer la conversion analogique-numrique
        while(BusyADC());       // Attendre que l'ADC ait termin la conversion
        Vmosfet = ReadADC();     // Lire le rsultat de la conversion
 
        //---Calculer et excuter la correction de courant
        if (Vbatt >= Vmosfet)    // Condition normale si le courant est de dcharge...
        {
            Ibatt = Vbatt - Vmosfet;         // Simple soustraction des tensions pour driver le courant. Tout est fonction des constantes connues.
            if (Ibatt > Ibatt_cible)        // Si le courant est trop fort compar  la cible
            {
                if (cycle_service > 0) cycle_service--;     // Diminuer le courant par un saut unique du cycle de service du PWM
            }
            else if (Ibatt < Ibatt_cible)   // Sinon, si le courant est insuffisant compar  la cible
            {
                if (cycle_service < 1023) cycle_service++;  // Augmenter le courant par un saut unique du cycle de service du PWM
            }
        }
        else                            // Condition anormale
        {
            cycle_service = 0;        // Couper la sortie du PWM.
        }
     }
}
